
package eu.dnetlib.dhp.oa.graph.dump.gcat;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import com.google.gson.Gson;

import okhttp3.*;

/**
 * Created by Alessia Bardi on 19/06/2020.
 *
 * @author Alessia Bardi
 */
public class GCatAPIClient {

	private static final Log log = LogFactory.getLog(GCatAPIClient.class);

	public static final int BULK_SIZE = 100;
	private String gcatBaseURL;
	private final String itemPath = "items";
	private String applicationToken;

	private static final MediaType MEDIA_TYPE_JSON = MediaType.parse("application/json; charset=utf-8");

	public GCatAPIClient() {
	}

	/**
	 * Publish the json as in the D4science catalogue as an item.
	 * 
	 * @param jsonMetadata the whole published json record
	 * @return the HTTP status code of the request
	 * @throws IOException
	 */

	public int publish(final String jsonMetadata) throws IOException, URISyntaxException {
		OkHttpClient httpCLient = new OkHttpClient();

		RequestBody body = RequestBody.create(jsonMetadata, MEDIA_TYPE_JSON);

		Request request = new Request.Builder()
			.url(getGcatBaseURL() + itemPath)
			.header("gcube-token", getApplicationToken())
			.addHeader("Content-Type", "application/json")
			.addHeader("Accept", "application/json")
			.post(body)
			.build();

		try (Response response = httpCLient.newCall(request).execute()) {
			if (log.isDebugEnabled()) {
				log.debug(response.code());
				System.out.println(response.code());
				log.debug(response.body().string());
			}
			if (!response.isSuccessful()) {

				throw new IOException("Unexpected code " + response);
			}

			return response.code();
		}

	}

	/**
	 * List items in the catalogue
	 * 
	 * @param offset offset
	 * @param limit limit
	 * @return list of catalogue item names
	 * @throws IOException
	 * @throws URISyntaxException
	 */
	public List<String> list(final int offset, final int limit) throws IOException, URISyntaxException {
		OkHttpClient httpClient = new OkHttpClient();

		Request request = new Request.Builder()
			.url(getGcatBaseURL() + itemPath + "?offset=" + offset + "&limit=" + limit)
			.header("gcube-token", getApplicationToken())
			.addHeader("Content-Type", "application/json")
			.addHeader("Accept", "application/json")
			.get()
			.build();

		try (Response response = httpClient.newCall(request).execute()) {
			int status = response.code();
			if (status >= 200 && status < 300) {
				String entity = response.body().string();
				return entity != null ? new Gson().fromJson(entity, List.class) : null;
			} else {
				throw new ClientProtocolException("Unexpected response status: " + status);
			}

		}
	}

	public boolean purge(final String resCatName) throws IOException, URISyntaxException {
		try (CloseableHttpClient client = HttpClients.createDefault()) {
			URIBuilder builder = new URIBuilder(getGcatBaseURL() + itemPath + "/" + resCatName)
				.addParameter("purge", "true");
			URI uri = builder.build();
			System.out.println(uri.toString());
			HttpDelete del = new HttpDelete(uri);
			del.setHeader("gcube-token", getApplicationToken());
			del.addHeader("Content-Type", "application/json");
			del.addHeader("Accept", "application/json");
			HttpResponse response = client.execute(del);
			if (log.isDebugEnabled()) {
				log.debug(response.getStatusLine());
			}
			return response.getStatusLine().getStatusCode() == HttpStatus.SC_NO_CONTENT;
		}
	}

	public int purgeAll() throws IOException, URISyntaxException {
		int count = 0;
		int deleted = 0;
		int failed = 0;
		List<String> list = list(0, BULK_SIZE);
		do {
			for (String itemName : list) {
				count++;
				if (purge(itemName))
					deleted++;
				else {
					failed++;
					log.warn("Deletion of item " + itemName + " failed");
				}
			}
			list = list(0, BULK_SIZE);
		} while (list.size() > 0);
		log.info(String.format("PurgeAll completed: total = %d; deleted = %d; failed = %d", count, deleted, failed));
		return deleted;
	}

	public String getGcatBaseURL() {
		return gcatBaseURL;
	}

	public void setGcatBaseURL(String gcatBaseURL) {
		this.gcatBaseURL = gcatBaseURL;
	}

	public String getApplicationToken() {
		return applicationToken;
	}

	public void setApplicationToken(String applicationToken) {
		this.applicationToken = applicationToken;
	}
}
